# -*- coding: utf-8 -*-

''' Manejo de archivos por parte de la interfaz de usuario. '''

from common.utils import read_text_file, write_text_file

def parse_GUI_conf_file (file_name):

    ''' Procesar el archivo de configuración de la interfaz y devolver en
        una estructura de datos los valores de la configuración.
        @param file_name: la ruta al archivo de configuración.
        @type file_name: L{str}
        @return: los valores de la configuración, o L{None} si hay errores.
        @rtype: L{dict} '''

    result = read_text_file (file_name)

    if result is not None:
        lines = result.split ('\n')
        fields = {}
        result = {}

        for i in lines:
            i = i.strip ()

            # Si la línea empezase por "#" sería un comentario y se ignoraría:
            if not i.startswith ('#'):
                parts = i.split ('=')

                if 2 == len (parts):
                    # Se normalizan ambas partes de la asignación:
                    left_part = parts [0].strip ().lower ()
                    right_part = parts [1].strip ().lower ()

                    if len (left_part) > 0 and len (right_part) > 0:

                        if left_part not in fields.keys ():
                            fields [left_part] = right_part
                        # La variable está duplicada; es un error:
                        else:
                            result = None

                    # Si falta alguna de las dos partes, la línea es errónea:
                    else:
                        result = None

                # Si no contiene un "=" ni es vacía la línea es errónea:
                elif '' != i:
                    result = None

        if result is not None:

            # Si no hay exactamente tres variables, el archivo no es correcto:
            if 3 != len (fields.keys ()):
                result = None

        if result is not None:

            if 'labels' in fields.keys ():

                if 'yes' == fields ['labels']:
                    result ['labels'] = True
                elif 'no' == fields ['labels']:
                    result ['labels'] = False
                # Si se le asignó cualquier otro valor, es un error:
                else:
                    result = None

            # Si la variable "labels" no está, es un error:
            else:
                result = None

        if result is not None:

            if 'refresh' in fields.keys ():

                if fields ['refresh'].isdigit ():
                    tmp = int (fields ['refresh'])

                    if tmp >= 1 and tmp <= 3600:
                        result ['refresh'] = tmp
                    else:
                        # Si escapa de este rango, es un error:
                        result = None

                else:
                    # Si no contiene solo dígitos, es un error:
                    result = None

            # Si la variable "refresh" no está, es un error:
            else:
                result = None

        if result is not None:

            if 'font' in fields.keys ():

                if fields ['font'].isdigit ():
                    tmp = int (fields ['font'])

                    if tmp >= 6 and tmp <= 30:
                        result ['font'] = tmp
                    else:
                        # Si escapa de este rango, es un error:
                        result = None

                else:
                    # Si no contiene solo dígitos, es un error:
                    result = None

            # Si la variable "refresh" no está, es un error:
            else:
                result = None

    return result

def dump_GUI_conf_file (file_name, config_data):

    ''' Escribir la configuración de la interfaz en el archivo de configuración.
        @param file_name: la ruta al archivo de configuración.
        @type file_name: L{str}
        @param config_data: la configuración actual de la aplicación.
        @type config_data: L{dict}
        @return: si ha habido éxito, o no.
        @rtype: L{bool} '''

    text = '\n# Mostrar texto en los botones de la barra de herramientas:\n'

    if config_data ['labels']:
        text = text + 'labels = yes\n\n'
    else:
        text = text + 'labels = no\n\n'

    text = text + '# Frecuencia de refresco, en segundos:\n'
    text = text + 'refresh = ' + str (config_data ['refresh']) + '\n\n'
    text = text + '# Tamaño de la fuente para las áreas de texto:\n'
    text = text + 'font = ' + str (config_data ['font']) + '\n\n'
    return write_text_file (file_name, text)

class InvalidINECodesFile (Exception):

    ''' Archivo de I{códigos INE} no válido. '''

    pass

class Town_name_finder:

    ''' Correspondencia entre I{códigos INE} y nombres de municipios. '''

    def __init__ (self, file_name):

        ''' Crear un objeto que pueda devolver el nombre de cualquier municipio.
            @param file_name: la ruta al archivo de códigos INE.
            @type file_name: L{str} '''

        result = read_text_file (file_name)

        if result is not None:
            lines = result.split ('\n')
            result = {}

            for i in lines:
                i = i.strip ()

                # Si la línea empezase por "#" sería un comentario y se ignoraría:
                if not i.startswith ('#'):
                    parts = i.split (';')

                    if 3 == len (parts):
                        # Se normalizan las tres partes:
                        prov_code = parts [0].strip ()
                        mun_code = parts [1].strip ()
                        mun_name = parts [2].strip ()

                        if prov_code.isdigit () and mun_code.isdigit () and len (mun_name) > 0:
                            key = (int (prov_code), int (mun_code))

                            if key not in result.keys ():
                                result [key] = mun_name
                            # La entrada está duplicada; es un error:
                            else:
                                result = None

                        # Si el formato (nº, nº, nombre) no es correcto, error al canto:
                        else:
                            result = None

                    # Si no contiene dos ";" ni es vacía la línea es errónea:
                    elif '' != i:
                        result = None

        if result is None:
            raise InvalidINECodesFile
        else:
            self.data = result

    def find (self, code):

        ''' Averiguar el nombre de un municipio, dado su código INE.
            @param code: un par I{(código_provincia, código_municipio)}.
            @type code: L{tuple}
            @return: el nombre del municipio, o L{None} si hay errores
                (por ejemplo, porque no se encuentra).
            @rtype: L{str} '''

        if code in self.data.keys ():
            return self.data [code]
        else:
            return None

# Fin del archivo.

